<?php declare(strict_types=1);

namespace KorElf\TranslateLaravel\DTO;

use KorElf\TranslateLaravel\Enums\TextType;
use KorElf\TranslateLaravel\Exceptions\RunTranslateDtoException;

final class RunTranslateDto
{
    /**
     * @var array [key => \KorElf\TranslateLaravel\DTO\Parameter]
     */
    private array $params = [];

    public function addParam(
        string   $key,
        string   $text,
        TextType $textType,
        string   $targetLanguageCode,
        ?string  $sourceLanguageCode = null,
        ?string  $driver = null
    ): self
    {
        if (isset($this->params[$key])) {
            throw new RunTranslateDtoException('Duplicate key: ' . $key);
        }

        $this->params[$key] = new Parameter(
            $text,
            $textType,
            $targetLanguageCode,
            $sourceLanguageCode,
            $driver,
        );

        return $this;
    }

    public function addParamHtml(
        string   $key,
        string   $text,
        string   $targetLanguageCode,
        ?string  $sourceLanguageCode = null,
        ?string  $driver = null
    ): self
    {
        return $this->addParam($key, $text, TextType::Html, $targetLanguageCode, $sourceLanguageCode, $driver);
    }

    public function addParamText(
        string   $key,
        string   $text,
        string   $targetLanguageCode,
        ?string  $sourceLanguageCode = null,
        ?string  $driver = null
    ): self
    {
        return $this->addParam($key, $text, TextType::Text, $targetLanguageCode, $sourceLanguageCode, $driver);
    }

    public function getParams(): array
    {
        return $this->params;
    }
}