<?php declare(strict_types=1);

namespace KorElf\TranslateLaravel\Translate;

use Illuminate\Contracts\Container\BindingResolutionException;
use Illuminate\Contracts\Foundation\Application;
use Illuminate\Support\Facades\Log;
use KorElf\TranslateLaravel\Contracts\Translate;
use KorElf\TranslateLaravel\DTO\Languages;

final class LogDriver implements Translate
{
    /**
     * @throws BindingResolutionException
     */
    public static function init(Application $app, array $config = []): Translate
    {
        return new self(
            $app->make(Log::class),
        );
    }

    public function __construct(
        private readonly Log $log
    ) { }

    public function translateText(array|string $text, string $targetLanguageCode, ?string $sourceLanguageCode = null): string|array
    {
        $this->log::info('Translate text', ['text' => $text, 'targetLanguageCode' => $targetLanguageCode, 'sourceLanguageCode' => $sourceLanguageCode]);

        return $text;
    }

    public function translateHtml(array|string $text, string $targetLanguageCode, ?string $sourceLanguageCode = null): string|array
    {
        $this->log::info('Translate html', ['text' => $text, 'targetLanguageCode' => $targetLanguageCode, 'sourceLanguageCode' => $sourceLanguageCode]);

        return $text;
    }

    public function listLanguages(): Languages
    {
        $languages = new Languages();
        $languages->add('ru', 'Russian');
        $languages->add('en', 'English');
        $languages->add('de', 'Deutsch');

        return $languages;
    }
}