<?php declare(strict_types=1);

namespace KorElf\TranslateLaravel;

use Illuminate\Cache\RateLimiting\Limit;
use Illuminate\Support\Facades\RateLimiter;
use Illuminate\Support\ServiceProvider;
use Illuminate\Contracts\Foundation\Application;
use KorElf\TranslateLaravel\Contracts\ProcessTranslateContract;
use KorElf\TranslateLaravel\Translate\TranslateManager;

final class TranslateLaravelProvider extends ServiceProvider
{
    public function boot(): Void
    {
        $this->publishes([
            __DIR__ . '/../config/translate.php' => config_path('translate.php'),
        ], 'config');

        RateLimiter::for('kor-elf-translate', function (ProcessTranslateContract $job) {
            $limited = $job->getLimited();

            return Limit::perSecond(
                $limited->getMaxRequest(),
                $limited->getRateSeconds()
            )->by($limited->getDriver());
        });
    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register(): Void
    {
        $this->mergeConfigFrom(
            __DIR__ . '/../config/translate.php',
            'translate'
        );
        $this->app->singleton('translate', function (Application $app) {
            return new TranslateManager($app);
        });
    }

}